@@ -7,11 +7,16 @@ import unittest
 
 # EXT
 import numpy as np
+import torch
+import tensorflow as tf
+
+# import jax.numpy as jnp
 from scipy.stats import wasserstein_distance, pearsonr
 
 # PKG
 from deepsig.aso import (
     aso,
+    multi_aso,
     compute_violation_ratio,
     get_quantile_function,
 )
@@ -109,15 +114,14 @@ class ASOTechnicalTests(unittest.TestCase):
 
         for (loc1, scale1), (loc2, scale2) in parameters:
             samples_normal1 = np.random.normal(
-                loc=loc1, scale=scale1, size=5
+                loc=loc1, scale=scale1, size=500
             )  # New scores for algorithm A
             samples_normal2 = np.random.normal(
-                loc=loc2, scale=scale2, size=5
+                loc=loc2, scale=scale2, size=500
             )  # Scores for algorithm B
 
             # Set seed to make sure that any variance just comes from the difference between jobs
             seed = np.random.randint(0, 10000)
-            np.random.seed(seed)
 
             eps_min1 = aso(
                 samples_normal1,
@@ -125,17 +129,96 @@ class ASOTechnicalTests(unittest.TestCase):
                 num_bootstrap_iterations=500,
                 num_jobs=1,
                 show_progress=False,
+                seed=seed,
             )
 
-            np.random.seed(seed)
             eps_min2 = aso(
                 samples_normal1,
                 samples_normal2,
                 num_bootstrap_iterations=500,
                 num_jobs=2,
                 show_progress=False,
+                seed=seed,
             )
-            self.assertAlmostEqual(eps_min1, eps_min2, delta=0.1)
+            self.assertAlmostEqual(eps_min1, eps_min2, delta=0.12)
+
+
+class MultiASOTests(unittest.TestCase):
+    """
+    Test different aspects of multi_aso().
+    """
+
+    def setUp(self) -> None:
+        self.aso_kwargs = {
+            "num_samples": 100,
+            "num_bootstrap_iterations": 100,
+            "num_jobs": 2,
+        }
+        self.num_models = 3
+        self.num_seeds = 100
+        self.scores = [
+            np.random.normal(loc=0.3, scale=0.2, size=self.num_seeds).tolist()
+            for _ in range(self.num_models)
+        ]
+        self.scores_dict = {
+            "model{}".format(i): scores for i, scores in enumerate(self.scores)
+        }
+        self.scores_numpy = np.array(self.scores)
+        self.scores_torch = torch.from_numpy(self.scores_numpy)
+        self.scores_tensorflow = tf.convert_to_tensor(self.scores_numpy)
+        self.all_score_types = [
+            self.scores,
+            self.scores_dict,
+            self.scores_numpy,
+            self.scores_torch,
+            self.scores_tensorflow,
+        ]
+        # self.scores_jax = jnp.array(self.scores_numpy)
+
+    def test_score_types(self):
+        """
+        Test different types for the scores argument.
+        """
+        for scores in self.all_score_types:
+            multi_aso(scores, **self.aso_kwargs)
+
+    def test_bonferroni_correction(self):
+        """
+        Test flag that toggles the use of the Bonferroni correction.
+        """
+        seed = 123
+        corrected_scores = multi_aso(self.scores_numpy, seed=seed, **self.aso_kwargs)
+        uncorrected_scores = multi_aso(
+            self.scores_numpy, seed=seed, use_bonferroni=False, **self.aso_kwargs
+        )
+        self.assertTrue(np.all(corrected_scores >= uncorrected_scores))
+
+    def test_symmetry(self):
+        """
+        Test flag that toggles the use of the symmetry property.
+        """
+        seed = 4321
+        asymmetric_scores = multi_aso(
+            self.scores_numpy, seed=seed, use_symmetry=False, **self.aso_kwargs
+        )
+        symmetric_scores = multi_aso(self.scores_numpy, seed=seed, **self.aso_kwargs)
+
+        self.assertTrue(np.all(symmetric_scores == symmetric_scores.T))
+        self.assertTrue(np.any(asymmetric_scores != asymmetric_scores.T))
+
+    def test_result_df(self):
+        """
+        Test the creation of a results DataFrame.
+        """
+        seed = 5555
+        eps_min = multi_aso(self.scores_dict, seed=seed, **self.aso_kwargs)
+        eps_min_df = multi_aso(
+            self.scores_dict, seed=seed, **self.aso_kwargs, return_df=True
+        )
+
+        self.assertEqual(list(self.scores_dict.keys()), list(eps_min_df.columns))
+        self.assertEqual(list(self.scores_dict.keys()), list(eps_min_df.index))
+        self.assertTrue(np.all(eps_min == eps_min_df.to_numpy()))
 
 
 class ASOSanityChecks(unittest.TestCase):
@@ -193,6 +276,7 @@ class ASOSanityChecks(unittest.TestCase):
         )  # Scores for algorithm B
 
         min_epsilons = []
+        seed = 6666
         for alpha in np.arange(0.8, 0.1, -0.1):
             min_eps = aso(
                 samples_normal1,
@@ -201,6 +285,7 @@ class ASOSanityChecks(unittest.TestCase):
                 num_bootstrap_iterations=100,
                 show_progress=False,
                 num_jobs=2,
+                seed=seed,
             )
             min_epsilons.append(min_eps)
 
@@ -213,6 +298,7 @@ class ASOSanityChecks(unittest.TestCase):
         Make sure that the minimum epsilon threshold decreases as we increase the number of samples.
         """
         min_epsilons = []
+        seed = 7890
 
         for num_samples in [80, 1000, 8000]:
             samples_normal2 = np.random.normal(
@@ -223,9 +309,10 @@ class ASOSanityChecks(unittest.TestCase):
             min_eps = aso(
                 samples_normal1,
                 samples_normal2,
-                num_bootstrap_iterations=10,
+                num_bootstrap_iterations=100,
                 show_progress=False,
                 num_jobs=2,
+                seed=seed,
             )
             min_epsilons.append(min_eps)
 
